#ifndef __ExpressionSimplifier_h__
#define __ExpressionSimplifier_h__


#include <string>
#include <list>
//#include <pair>
#include <ginac/ginac.h>


class ExpressionSimplifier:
	public GiNaC::visitor,
	public GiNaC::numeric::visitor,
	public GiNaC::constant::visitor,
	public GiNaC::symbol::visitor,
	public GiNaC::power::visitor,
	public GiNaC::add::visitor,
	public GiNaC::mul::visitor,
	public GiNaC::function::visitor,
	public GiNaC::basic::visitor
{
public:
	typedef std::pair<GiNaC::symbol,GiNaC::ex> symexpair;
private:
	// helper functions
	const GiNaC::symbol & getNextSymbol();
	GiNaC::symbol registerExpression(const GiNaC::ex & e);
	inline const GiNaC::ex & doVisit(const GiNaC::ex & e);

	// major datastructures
	GiNaC::exhashmap<GiNaC::symbol> exToSym;
	GiNaC::exmap                    symToEx;
	std::list<symexpair>            symExList;
	
	// other internal data
	std::string tempsymbolbasename;
	int         lasttempsymbolnumber;
	GiNaC::ex   result;
public:
	ExpressionSimplifier(const std::string & tempsymbolbasename="_s", int initialsymbolnumber=0);

	void visit(const GiNaC::numeric & e);
	void visit(const GiNaC::constant & e);
	void visit(const GiNaC::symbol & e);
	void visit(const GiNaC::power & e);
	void visit(const GiNaC::add & e);
	void visit(const GiNaC::mul & e);
	void visit(const GiNaC::function & e);
	void visit(const GiNaC::basic & e);

	GiNaC::exhashmap<GiNaC::symbol> & getExToSym()    { return exToSym; }
	GiNaC::exmap                    & getSymToEx()    { return symToEx; }
	std::list<symexpair>            & getSymExList()  { return symExList; }

	GiNaC::ex simplify(GiNaC::ex & e);
};


#endif 
