
#ifndef POLYGON_IS_INCLUDED
#define POLYGON_IS_INCLUDED

#include "tools.h"

namespace SyFi {

class Polygon; 
class Triangle; 
class Mesh; 
class Point; 
class Edge; 
class Face; 

enum  Repr_format { 
    SUBS_PERFORMED = 1, 
    SUBS_NOT_PERFORMED = 2
}; 

 

class Polygon {
    public: 
        string subscript; 
        GiNaC::exvector p; 

        Polygon() {}
        virtual ~Polygon() {}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual GiNaC::ex integrate(GiNaC::ex f, Repr_format format = SUBS_PERFORMED) { 
            cout <<"Polygon integrate not implemented "<<endl; 
            return DUMMY;  
        }    
        virtual string str(); 
}; 

class Line : public Polygon  {
    GiNaC::ex a_; 
    GiNaC::ex b_; 
    public: 
        Line() {}
        Line(GiNaC::ex x0, GiNaC::ex x1, string subscript = ""); // x0_ and x1_ are points 
        virtual ~Line(){}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual GiNaC::ex repr(GiNaC::ex t = GiNaC::symbol("t"), Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f, Repr_format format = SUBS_PERFORMED);    
}; 


class ReferenceLine : public Line {
    public: 
        ReferenceLine(string subscript = ""); 
        virtual ~ReferenceLine(){}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual GiNaC::ex repr(GiNaC::ex t, Repr_format format = SUBS_PERFORMED); 
        GiNaC::ex a();  
        GiNaC::ex b(); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f, Repr_format format = SUBS_PERFORMED); 
}; 

class Triangle : public Polygon {
    public: 
        Triangle(GiNaC::ex x0, GiNaC::ex x1, GiNaC::ex x1, string subscript = ""); 
        Triangle() {}
        virtual ~Triangle(){}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual GiNaC::ex repr(Repr_format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f, Repr_format format = SUBS_PERFORMED); 
}; 

class ReferenceTriangle : public Triangle {
    public: 
        ReferenceTriangle(string subscript = ""); 
        virtual ~ReferenceTriangle(){}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f); 
}; 

class Rectangle : public Polygon {
    public: 
        Rectangle(GiNaC::ex p0, GiNaC::ex p1, string subscript = "");
        Rectangle() {} 
        virtual ~Rectangle(){} 

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f); 
}; 



class ReferenceRectangle : public Rectangle {
    public: 
        ReferenceRectangle(string subscript = "");
        virtual ~ReferenceRectangle(){} 

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f); 
}; 

class Tetrahedron : public Polygon {
    public: 
        Tetrahedron(string subscript) {}
        Tetrahedron(GiNaC::ex x0, GiNaC::ex x1, GiNaC::ex x1, GiNaC::ex x2, string subscript = ""); 
        virtual ~Tetrahedron(){}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual Triangle triangle(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f, Repr_format format = SUBS_PERFORMED);
}; 

class ReferenceTetrahedron : public Tetrahedron {
    public: 
        ReferenceTetrahedron(string subscript = ""); 
        virtual ~ReferenceTetrahedron(){}

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual Triangle triangle(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED) { return GiNaC::ex(0); } 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f, Repr_format = SUBS_PERFORMED);  
}; 

class Box: public Polygon {
    public: 
        Box(GiNaC::ex p0, GiNaC::ex p1, string subscript = "");
        Box(){}
        virtual ~Box(){} 

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f); 
}; 



class ReferenceBox: public Box{
    public: 
        ReferenceBox(string subscript = "");
        virtual ~ReferenceBox(){} 

        virtual int no_vertices() const; 
	virtual int no_space_dim() const; 
        virtual GiNaC::ex vertex(int i); 
        virtual Line line(int i); 
        virtual GiNaC::ex repr(Repr_format format = SUBS_PERFORMED); 
        virtual string str(); 
        virtual GiNaC::ex integrate(GiNaC::ex f); 
}; 

// Some tools for Polygons 

// FIXME: change to barycenter(Triangle&)  
// FIXME:       and barycenter(Tetrahedron&)  
GiNaC::ex barycenter_line(GiNaC::ex p0, GiNaC::ex p1); 
GiNaC::ex barycenter_triangle(GiNaC::ex p0, GiNaC::ex p1, GiNaC::ex p2); 
GiNaC::ex barycenter_tetrahedron(GiNaC::ex p0, GiNaC::ex p1, GiNaC::ex p2, GiNaC::ex p3); 
GiNaC::lst bezier_ordinates(Triangle& triangle, int d);  
GiNaC::lst bezier_ordinates(Tetrahedron& tetrahedra, int d);  

// polynom of arbitrary order on a line, a triangle, 
// or a tetrahedron using the Bernstein basis 
GiNaC::ex bernstein(int order, Polygon& p, const string a);  

// vector polynom of arbitrary order on a line, a triangle, 
// or a tetrahedron using the Bernstein basis 
GiNaC::lst bernsteinv(int no_fields, int order, Polygon& p, const string a);  

GiNaC::lst normal(Triangle&, int i); 
GiNaC::lst normal(Tetrahedron&, int i); 

GiNaC::lst tangent(Triangle&, int i); 


}



#endif

